@@ -1,21 +1,25 @@
 mod completion;
-mod db;
 mod docstring;
 mod find_node;
 mod goto;
+mod goto_declaration;
+mod goto_definition;
+mod goto_type_definition;
 mod hover;
 mod inlay_hints;
 mod markup;
+mod references;
 mod semantic_tokens;
 mod signature_help;
+mod stub_mapping;
 
 pub use completion::completion;
-pub use db::Db;
 pub use docstring::get_parameter_documentation;
-pub use goto::goto_type_definition;
+pub use goto::{goto_declaration, goto_definition, goto_type_definition};
 pub use hover::hover;
 pub use inlay_hints::inlay_hints;
 pub use markup::MarkupKind;
+pub use references::references;
 pub use semantic_tokens::{
     SemanticToken, SemanticTokenModifier, SemanticTokenType, SemanticTokens, semantic_tokens,
 };
@@ -25,6 +29,7 @@ use ruff_db::files::{File, FileRange};
 use ruff_text_size::{Ranged, TextRange};
 use rustc_hash::FxHashSet;
 use std::ops::{Deref, DerefMut};
+use ty_project::Db;
 use ty_python_semantic::types::{Type, TypeDefinition};
 
 /// Information associated with a text range.
@@ -83,6 +88,15 @@ pub struct NavigationTarget {
 }
 
 impl NavigationTarget {
+    /// Creates a new `NavigationTarget` where the focus and full range are identical.
+    pub fn new(file: File, range: TextRange) -> Self {
+        Self {
+            file,
+            focus_range: range,
+            full_range: range,
+        }
+    }
+
     pub fn file(&self) -> File {
         self.file
     }
@@ -101,11 +115,11 @@ pub struct NavigationTargets(smallvec::SmallVec<[NavigationTarget; 1]>);
 
 impl NavigationTargets {
     fn single(target: NavigationTarget) -> Self {
-        Self(smallvec::smallvec![target])
+        Self(smallvec::smallvec_inline![target])
     }
 
     fn empty() -> Self {
-        Self(smallvec::SmallVec::new())
+        Self(smallvec::SmallVec::new_const())
     }
 
     fn unique(targets: impl IntoIterator<Item = NavigationTarget>) -> Self {
@@ -207,13 +221,13 @@ impl HasNavigationTargets for TypeDefinition<'_> {
 
 #[cfg(test)]
 mod tests {
-    use crate::db::tests::TestDb;
     use insta::internals::SettingsBindDropGuard;
     use ruff_db::Db;
     use ruff_db::diagnostic::{Diagnostic, DiagnosticFormat, DisplayDiagnosticConfig};
     use ruff_db::files::{File, system_path_to_file};
     use ruff_db::system::{DbWithWritableSystem, SystemPath, SystemPathBuf};
     use ruff_text_size::TextSize;
+    use ty_project::ProjectMetadata;
     use ty_python_semantic::{
         Program, ProgramSettings, PythonPlatform, PythonVersionWithSource, SearchPathSettings,
     };
@@ -227,7 +241,7 @@ mod tests {
     }
 
     pub(super) struct CursorTest {
-        pub(super) db: TestDb,
+        pub(super) db: ty_project::TestDb,
         pub(super) cursor: Cursor,
         _insta_settings_guard: SettingsBindDropGuard,
     }
@@ -282,8 +296,13 @@ mod tests {
 
     impl CursorTestBuilder {
         pub(super) fn build(&self) -> CursorTest {
-            let mut db = TestDb::new();
+            let mut db = ty_project::TestDb::new(ProjectMetadata::new(
+                "test".into(),
+                SystemPathBuf::from("/"),
+            ));
+
             let mut cursor: Option<Cursor> = None;
+
             for &Source {
                 ref path,
                 ref contents,
@@ -292,19 +311,19 @@ mod tests {
             {
                 db.write_file(path, contents)
                     .expect("write to memory file system to be successful");
-                let Some(offset) = cursor_offset else {
-                    continue;
-                };
 
                 let file = system_path_to_file(&db, path).expect("newly written file to existing");
-                // This assert should generally never trip, since
-                // we have an assert on `CursorTestBuilder::source`
-                // to ensure we never have more than one marker.
-                assert!(
-                    cursor.is_none(),
-                    "found more than one source that contains `<CURSOR>`"
-                );
-                cursor = Some(Cursor { file, offset });
+
+                if let Some(offset) = cursor_offset {
+                    // This assert should generally never trip, since
+                    // we have an assert on `CursorTestBuilder::source`
+                    // to ensure we never have more than one marker.
+                    assert!(
+                        cursor.is_none(),
+                        "found more than one source that contains `<CURSOR>`"
+                    );
+                    cursor = Some(Cursor { file, offset });
+                }
             }
 
             let search_paths = SearchPathSettings::new(vec![SystemPathBuf::from("/")])
