@@ -8,7 +8,7 @@ use rustc_hash::{FxBuildHasher, FxHashSet};
 
 use ruff_db::files::{File, FilePath, FileRootKind};
 use ruff_db::system::{DirectoryEntry, System, SystemPath, SystemPathBuf};
-use ruff_db::vendored::{VendoredFileSystem, VendoredPath};
+use ruff_db::vendored::VendoredFileSystem;
 use ruff_python_ast::PythonVersion;
 
 use crate::db::Db;
@@ -17,15 +17,36 @@ use crate::module_resolver::typeshed::{TypeshedVersions, vendored_typeshed_versi
 use crate::{Program, SearchPathSettings};
 
 use super::module::{Module, ModuleKind};
-use super::path::{ModulePath, SearchPath, SearchPathValidationError};
+use super::path::{ModulePath, SearchPath, SearchPathValidationError, SystemOrVendoredPathRef};
 
 /// Resolves a module name to a module.
-pub fn resolve_module(db: &dyn Db, module_name: &ModuleName) -> Option<Module> {
-    let interned_name = ModuleNameIngredient::new(db, module_name);
+pub fn resolve_module<'db>(db: &'db dyn Db, module_name: &ModuleName) -> Option<Module<'db>> {
+    let interned_name = ModuleNameIngredient::new(db, module_name, ModuleResolveMode::StubsAllowed);
 
     resolve_module_query(db, interned_name)
 }
 
+/// Resolves a module name to a module (stubs not allowed).
+pub fn resolve_real_module<'db>(db: &'db dyn Db, module_name: &ModuleName) -> Option<Module<'db>> {
+    let interned_name =
+        ModuleNameIngredient::new(db, module_name, ModuleResolveMode::StubsNotAllowed);
+
+    resolve_module_query(db, interned_name)
+}
+
+/// Which files should be visible when doing a module query
+#[derive(Debug, Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash)]
+pub(crate) enum ModuleResolveMode {
+    StubsAllowed,
+    StubsNotAllowed,
+}
+
+impl ModuleResolveMode {
+    fn stubs_allowed(self) -> bool {
+        matches!(self, Self::StubsAllowed)
+    }
+}
+
 /// Salsa query that resolves an interned [`ModuleNameIngredient`] to a module.
 ///
 /// This query should not be called directly. Instead, use [`resolve_module`]. It only exists
@@ -34,11 +55,12 @@ pub fn resolve_module(db: &dyn Db, module_name: &ModuleName) -> Option<Module> {
 pub(crate) fn resolve_module_query<'db>(
     db: &'db dyn Db,
     module_name: ModuleNameIngredient<'db>,
-) -> Option<Module> {
+) -> Option<Module<'db>> {
     let name = module_name.name(db);
+    let mode = module_name.mode(db);
     let _span = tracing::trace_span!("resolve_module", %name).entered();
 
-    let Some(resolved) = resolve_name(db, name) else {
+    let Some(resolved) = resolve_name(db, name, mode) else {
         tracing::debug!("Module `{name}` not found in search paths");
         return None;
     };
@@ -49,11 +71,17 @@ pub(crate) fn resolve_module_query<'db>(
                 "Resolved module `{name}` to `{path}`",
                 path = module.file.path(db)
             );
-            Module::file_module(name.clone(), module.kind, module.search_path, module.file)
+            Module::file_module(
+                db,
+                name.clone(),
+                module.kind,
+                module.search_path,
+                module.file,
+            )
         }
         ResolvedName::NamespacePackage => {
             tracing::trace!("Module `{name}` is a namespace package");
-            Module::namespace_package(name.clone())
+            Module::namespace_package(db, name.clone())
         }
     };
 
@@ -64,7 +92,7 @@ pub(crate) fn resolve_module_query<'db>(
 ///
 /// Returns `None` if the path is not a module locatable via any of the known search paths.
 #[allow(unused)]
-pub(crate) fn path_to_module(db: &dyn Db, path: &FilePath) -> Option<Module> {
+pub(crate) fn path_to_module<'db>(db: &'db dyn Db, path: &FilePath) -> Option<Module<'db>> {
     // It's not entirely clear on first sight why this method calls `file_to_module` instead of
     // it being the other way round, considering that the first thing that `file_to_module` does
     // is to retrieve the file's path.
@@ -77,33 +105,14 @@ pub(crate) fn path_to_module(db: &dyn Db, path: &FilePath) -> Option<Module> {
     file_to_module(db, file)
 }
 
-#[derive(Debug, Clone, Copy)]
-enum SystemOrVendoredPathRef<'a> {
-    System(&'a SystemPath),
-    Vendored(&'a VendoredPath),
-}
-
-impl std::fmt::Display for SystemOrVendoredPathRef<'_> {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        match self {
-            SystemOrVendoredPathRef::System(system) => system.fmt(f),
-            SystemOrVendoredPathRef::Vendored(vendored) => vendored.fmt(f),
-        }
-    }
-}
-
 /// Resolves the module for the file with the given id.
 ///
 /// Returns `None` if the file is not a module locatable via any of the known search paths.
 #[salsa::tracked(heap_size=get_size2::GetSize::get_heap_size)]
-pub(crate) fn file_to_module(db: &dyn Db, file: File) -> Option<Module> {
+pub(crate) fn file_to_module(db: &dyn Db, file: File) -> Option<Module<'_>> {
     let _span = tracing::trace_span!("file_to_module", ?file).entered();
 
-    let path = match file.path(db) {
-        FilePath::System(system) => SystemOrVendoredPathRef::System(system),
-        FilePath::Vendored(vendored) => SystemOrVendoredPathRef::Vendored(vendored),
-        FilePath::SystemVirtual(_) => return None,
-    };
+    let path = SystemOrVendoredPathRef::try_from_file(db, file)?;
 
     let module_name = search_paths(db).find_map(|candidate| {
         let relative_path = match path {
@@ -118,7 +127,7 @@ pub(crate) fn file_to_module(db: &dyn Db, file: File) -> Option<Module> {
     // root paths, but that the module corresponding to `path` is in a lower priority search path,
     // in which case we ignore it.
     let module = resolve_module(db, &module_name)?;
-    let module_file = module.file()?;
+    let module_file = module.file(db)?;
 
     if file.path(db) == module_file.path(db) {
         Some(module)
@@ -533,16 +542,26 @@ impl<'db> Iterator for PthFileIterator<'db> {
 struct ModuleNameIngredient<'db> {
     #[returns(ref)]
     pub(super) name: ModuleName,
+    pub(super) mode: ModuleResolveMode,
+}
+
+/// Returns `true` if the module name refers to a standard library module which can't be shadowed
+/// by a first-party module.
+///
+/// This includes "builtin" modules, which can never be shadowed at runtime either, as well as the
+/// `types` module, which tends to be imported early in Python startup, so can't be consistently
+/// shadowed, and is important to type checking.
+fn is_non_shadowable(minor_version: u8, module_name: &str) -> bool {
+    module_name == "types" || ruff_python_stdlib::sys::is_builtin_module(minor_version, module_name)
 }
 
 /// Given a module name and a list of search paths in which to lookup modules,
 /// attempt to resolve the module name
-fn resolve_name(db: &dyn Db, name: &ModuleName) -> Option<ResolvedName> {
+fn resolve_name(db: &dyn Db, name: &ModuleName, mode: ModuleResolveMode) -> Option<ResolvedName> {
     let program = Program::get(db);
     let python_version = program.python_version(db);
-    let resolver_state = ResolverContext::new(db, python_version);
-    let is_builtin_module =
-        ruff_python_stdlib::sys::is_builtin_module(python_version.minor, name.as_str());
+    let resolver_state = ResolverContext::new(db, python_version, mode);
+    let is_non_shadowable = is_non_shadowable(python_version.minor, name.as_str());
 
     let name = RelaxedModuleName::new(name);
     let stub_name = name.to_stub_package();
@@ -553,11 +572,12 @@ fn resolve_name(db: &dyn Db, name: &ModuleName) -> Option<ResolvedName> {
         // the module name always resolves to the stdlib module,
         // even if there's a module of the same name in the first-party root
         // (which would normally result in the stdlib module being overridden).
-        if is_builtin_module && !search_path.is_standard_library() {
+        // TODO: offer a diagnostic if there is a first-party module of the same name
+        if is_non_shadowable && !search_path.is_standard_library() {
             continue;
         }
 
-        if !search_path.is_standard_library() {
+        if !search_path.is_standard_library() && resolver_state.mode.stubs_allowed() {
             match resolve_name_in_search_path(&resolver_state, &stub_name, search_path) {
                 Ok((package_kind, ResolvedName::FileModule(module))) => {
                     if package_kind.is_root() && module.kind.is_module() {
@@ -726,14 +746,16 @@ fn resolve_name_in_search_path(
 /// resolving modules.
 fn resolve_file_module(module: &ModulePath, resolver_state: &ResolverContext) -> Option<File> {
     // Stubs have precedence over source files
-    let file = module
-        .with_pyi_extension()
-        .to_file(resolver_state)
-        .or_else(|| {
-            module
-                .with_py_extension()
-                .and_then(|path| path.to_file(resolver_state))
-        })?;
+    let stub_file = if resolver_state.mode.stubs_allowed() {
+        module.with_pyi_extension().to_file(resolver_state)
+    } else {
+        None
+    };
+    let file = stub_file.or_else(|| {
+        module
+            .with_py_extension()
+            .and_then(|path| path.to_file(resolver_state))
+    })?;
 
     // For system files, test if the path has the correct casing.
     // We can skip this step for vendored files or virtual files because
@@ -842,11 +864,20 @@ impl PackageKind {
 pub(super) struct ResolverContext<'db> {
     pub(super) db: &'db dyn Db,
     pub(super) python_version: PythonVersion,
+    pub(super) mode: ModuleResolveMode,
 }
 
 impl<'db> ResolverContext<'db> {
-    pub(super) fn new(db: &'db dyn Db, python_version: PythonVersion) -> Self {
-        Self { db, python_version }
+    pub(super) fn new(
+        db: &'db dyn Db,
+        python_version: PythonVersion,
+        mode: ModuleResolveMode,
+    ) -> Self {
+        Self {
+            db,
+            python_version,
+            mode,
+        }
     }
 
     pub(super) fn vendored(&self) -> &VendoredFileSystem {
@@ -914,12 +945,12 @@ mod tests {
             resolve_module(&db, &foo_module_name).as_ref()
         );
 
-        assert_eq!("foo", foo_module.name());
-        assert_eq!(&src, foo_module.search_path().unwrap());
-        assert_eq!(ModuleKind::Module, foo_module.kind());
+        assert_eq!("foo", foo_module.name(&db));
+        assert_eq!(&src, foo_module.search_path(&db).unwrap());
+        assert_eq!(ModuleKind::Module, foo_module.kind(&db));
 
         let expected_foo_path = src.join("foo.py");
-        assert_eq!(&expected_foo_path, foo_module.file().unwrap().path(&db));
+        assert_eq!(&expected_foo_path, foo_module.file(&db).unwrap().path(&db));
         assert_eq!(
             Some(foo_module),
             path_to_module(&db, &FilePath::System(expected_foo_path))
@@ -937,7 +968,7 @@ mod tests {
         let builtins = resolve_module(&db, &builtins_module_name).expect("builtins to resolve");
 
         assert_eq!(
-            builtins.file().unwrap().path(&db),
+            builtins.file(&db).unwrap().path(&db),
             &stdlib.join("builtins.pyi")
         );
     }
@@ -961,7 +992,7 @@ mod tests {
         let builtins = resolve_module(&db, &builtins_module_name).expect("builtins to resolve");
 
         assert_eq!(
-            builtins.file().unwrap().path(&db),
+            builtins.file(&db).unwrap().path(&db),
             &stdlib.join("builtins.pyi")
         );
     }
@@ -986,13 +1017,13 @@ mod tests {
             resolve_module(&db, &functools_module_name).as_ref()
         );
 
-        assert_eq!(&stdlib, functools_module.search_path().unwrap());
-        assert_eq!(ModuleKind::Module, functools_module.kind());
+        assert_eq!(&stdlib, functools_module.search_path(&db).unwrap());
+        assert_eq!(ModuleKind::Module, functools_module.kind(&db));
 
         let expected_functools_path = stdlib.join("functools.pyi");
         assert_eq!(
             &expected_functools_path,
-            functools_module.file().unwrap().path(&db)
+            functools_module.file(&db).unwrap().path(&db)
         );
 
         assert_eq!(
@@ -1039,7 +1070,7 @@ mod tests {
             let resolved_module = resolve_module(&db, &module_name).unwrap_or_else(|| {
                 panic!("Expected module {module_name} to exist in the mock stdlib")
             });
-            let search_path = resolved_module.search_path().unwrap();
+            let search_path = resolved_module.search_path(&db).unwrap();
             assert_eq!(
                 &stdlib, search_path,
                 "Search path for {module_name} was unexpectedly {search_path:?}"
@@ -1135,7 +1166,7 @@ mod tests {
             let resolved_module = resolve_module(&db, &module_name).unwrap_or_else(|| {
                 panic!("Expected module {module_name} to exist in the mock stdlib")
             });
-            let search_path = resolved_module.search_path().unwrap();
+            let search_path = resolved_module.search_path(&db).unwrap();
             assert_eq!(
                 &stdlib, search_path,
                 "Search path for {module_name} was unexpectedly {search_path:?}"
@@ -1196,11 +1227,11 @@ mod tests {
             Some(&functools_module),
             resolve_module(&db, &functools_module_name).as_ref()
         );
-        assert_eq!(&src, functools_module.search_path().unwrap());
-        assert_eq!(ModuleKind::Module, functools_module.kind());
+        assert_eq!(&src, functools_module.search_path(&db).unwrap());
+        assert_eq!(ModuleKind::Module, functools_module.kind(&db));
         assert_eq!(
             &src.join("functools.py"),
-            functools_module.file().unwrap().path(&db)
+            functools_module.file(&db).unwrap().path(&db)
         );
 
         assert_eq!(
@@ -1219,10 +1250,10 @@ mod tests {
         let pydoc_data_topics_name = ModuleName::new_static("pydoc_data.topics").unwrap();
         let pydoc_data_topics = resolve_module(&db, &pydoc_data_topics_name).unwrap();
 
-        assert_eq!("pydoc_data.topics", pydoc_data_topics.name());
-        assert_eq!(pydoc_data_topics.search_path().unwrap(), &stdlib);
+        assert_eq!("pydoc_data.topics", pydoc_data_topics.name(&db));
+        assert_eq!(pydoc_data_topics.search_path(&db).unwrap(), &stdlib);
         assert_eq!(
-            pydoc_data_topics.file().unwrap().path(&db),
+            pydoc_data_topics.file(&db).unwrap().path(&db),
             &stdlib.join("pydoc_data/topics.pyi")
         );
     }
@@ -1236,9 +1267,9 @@ mod tests {
         let foo_path = src.join("foo/__init__.py");
         let foo_module = resolve_module(&db, &ModuleName::new_static("foo").unwrap()).unwrap();
 
-        assert_eq!("foo", foo_module.name());
-        assert_eq!(&src, foo_module.search_path().unwrap());
-        assert_eq!(&foo_path, foo_module.file().unwrap().path(&db));
+        assert_eq!("foo", foo_module.name(&db));
+        assert_eq!(&src, foo_module.search_path(&db).unwrap());
+        assert_eq!(&foo_path, foo_module.file(&db).unwrap().path(&db));
 
         assert_eq!(
             Some(&foo_module),
@@ -1264,9 +1295,9 @@ mod tests {
         let foo_module = resolve_module(&db, &ModuleName::new_static("foo").unwrap()).unwrap();
         let foo_init_path = src.join("foo/__init__.py");
 
-        assert_eq!(&src, foo_module.search_path().unwrap());
-        assert_eq!(&foo_init_path, foo_module.file().unwrap().path(&db));
-        assert_eq!(ModuleKind::Package, foo_module.kind());
+        assert_eq!(&src, foo_module.search_path(&db).unwrap());
+        assert_eq!(&foo_init_path, foo_module.file(&db).unwrap().path(&db));
+        assert_eq!(ModuleKind::Package, foo_module.kind(&db));
 
         assert_eq!(
             Some(foo_module),
@@ -1287,8 +1318,8 @@ mod tests {
         let foo = resolve_module(&db, &ModuleName::new_static("foo").unwrap()).unwrap();
         let foo_stub = src.join("foo.pyi");
 
-        assert_eq!(&src, foo.search_path().unwrap());
-        assert_eq!(&foo_stub, foo.file().unwrap().path(&db));
+        assert_eq!(&src, foo.search_path(&db).unwrap());
+        assert_eq!(&foo_stub, foo.file(&db).unwrap().path(&db));
 
         assert_eq!(Some(foo), path_to_module(&db, &FilePath::System(foo_stub)));
         assert_eq!(
@@ -1311,8 +1342,8 @@ mod tests {
             resolve_module(&db, &ModuleName::new_static("foo.bar.baz").unwrap()).unwrap();
         let baz_path = src.join("foo/bar/baz.py");
 
-        assert_eq!(&src, baz_module.search_path().unwrap());
-        assert_eq!(&baz_path, baz_module.file().unwrap().path(&db));
+        assert_eq!(&src, baz_module.search_path(&db).unwrap());
+        assert_eq!(&baz_path, baz_module.file(&db).unwrap().path(&db));
 
         assert_eq!(
             Some(baz_module),
@@ -1335,8 +1366,8 @@ mod tests {
         let foo_module = resolve_module(&db, &ModuleName::new_static("foo").unwrap()).unwrap();
         let foo_src_path = src.join("foo.py");
 
-        assert_eq!(&src, foo_module.search_path().unwrap());
-        assert_eq!(&foo_src_path, foo_module.file().unwrap().path(&db));
+        assert_eq!(&src, foo_module.search_path(&db).unwrap());
+        assert_eq!(&foo_src_path, foo_module.file(&db).unwrap().path(&db));
         assert_eq!(
             Some(foo_module),
             path_to_module(&db, &FilePath::System(foo_src_path))
@@ -1408,14 +1439,14 @@ mod tests {
 
         assert_ne!(foo_module, bar_module);
 
-        assert_eq!(&src, foo_module.search_path().unwrap());
-        assert_eq!(&foo, foo_module.file().unwrap().path(&db));
+        assert_eq!(&src, foo_module.search_path(&db).unwrap());
+        assert_eq!(&foo, foo_module.file(&db).unwrap().path(&db));
 
         // `foo` and `bar` shouldn't resolve to the same file
 
-        assert_eq!(&src, bar_module.search_path().unwrap());
-        assert_eq!(&bar, bar_module.file().unwrap().path(&db));
-        assert_eq!(&foo, foo_module.file().unwrap().path(&db));
+        assert_eq!(&src, bar_module.search_path(&db).unwrap());
+        assert_eq!(&bar, bar_module.file(&db).unwrap().path(&db));
+        assert_eq!(&foo, foo_module.file(&db).unwrap().path(&db));
 
         assert_ne!(&foo_module, &bar_module);
 
@@ -1440,6 +1471,13 @@ mod tests {
 
         let foo_module_name = ModuleName::new_static("foo").unwrap();
         let foo_module = resolve_module(&db, &foo_module_name).unwrap();
+        let foo_pieces = (
+            foo_module.name(&db).clone(),
+            foo_module.file(&db),
+            foo_module.known(&db),
+            foo_module.search_path(&db).cloned(),
+            foo_module.kind(&db),
+        );
 
         let bar_path = src.join("bar.py");
         let bar = system_path_to_file(&db, &bar_path).expect("bar.py to exist");
@@ -1454,6 +1492,15 @@ mod tests {
         // for resolving `foo`.
 
         let foo_module2 = resolve_module(&db, &foo_module_name);
+        let foo_pieces2 = foo_module2.map(|foo_module2| {
+            (
+                foo_module2.name(&db).clone(),
+                foo_module2.file(&db),
+                foo_module2.known(&db),
+                foo_module2.search_path(&db).cloned(),
+                foo_module2.kind(&db),
+            )
+        });
 
         assert!(
             !db.take_salsa_events()
@@ -1461,7 +1508,7 @@ mod tests {
                 .any(|event| { matches!(event.kind, salsa::EventKind::WillExecute { .. }) })
         );
 
-        assert_eq!(Some(foo_module), foo_module2);
+        assert_eq!(Some(foo_pieces), foo_pieces2);
     }
 
     #[test]
@@ -1479,7 +1526,7 @@ mod tests {
         let foo_file = system_path_to_file(&db, &foo_path).expect("foo.py to exist");
 
         let foo_module = resolve_module(&db, &foo_module_name).expect("Foo module to resolve");
-        assert_eq!(foo_file, foo_module.file().unwrap());
+        assert_eq!(foo_file, foo_module.file(&db).unwrap());
 
         Ok(())
     }
@@ -1495,7 +1542,7 @@ mod tests {
         let foo_module = resolve_module(&db, &foo_module_name).expect("foo module to exist");
         let foo_init_path = src.join("foo/__init__.py");
 
-        assert_eq!(&foo_init_path, foo_module.file().unwrap().path(&db));
+        assert_eq!(&foo_init_path, foo_module.file(&db).unwrap().path(&db));
 
         // Delete `foo/__init__.py` and the `foo` folder. `foo` should now resolve to `foo.py`
         db.memory_file_system().remove_file(&foo_init_path)?;
@@ -1505,7 +1552,7 @@ mod tests {
         File::sync_path(&mut db, foo_init_path.parent().unwrap());
 
         let foo_module = resolve_module(&db, &foo_module_name).expect("Foo module to resolve");
-        assert_eq!(&src.join("foo.py"), foo_module.file().unwrap().path(&db));
+        assert_eq!(&src.join("foo.py"), foo_module.file(&db).unwrap().path(&db));
 
         Ok(())
     }
@@ -1531,9 +1578,9 @@ mod tests {
         let stdlib_functools_path = stdlib.join("functools.pyi");
 
         let functools_module = resolve_module(&db, &functools_module_name).unwrap();
-        assert_eq!(functools_module.search_path().unwrap(), &stdlib);
+        assert_eq!(functools_module.search_path(&db).unwrap(), &stdlib);
         assert_eq!(
-            Ok(functools_module.file().unwrap()),
+            Ok(functools_module.file(&db).unwrap()),
             system_path_to_file(&db, &stdlib_functools_path)
         );
 
@@ -1544,16 +1591,18 @@ mod tests {
         db.write_file(&site_packages_functools_path, "f: int")
             .unwrap();
         let functools_module = resolve_module(&db, &functools_module_name).unwrap();
+        let functools_file = functools_module.file(&db).unwrap();
+        let functools_search_path = functools_module.search_path(&db).unwrap().clone();
         let events = db.take_salsa_events();
         assert_function_query_was_not_run(
             &db,
             resolve_module_query,
-            ModuleNameIngredient::new(&db, functools_module_name),
+            ModuleNameIngredient::new(&db, functools_module_name, ModuleResolveMode::StubsAllowed),
             &events,
         );
-        assert_eq!(functools_module.search_path().unwrap(), &stdlib);
+        assert_eq!(&functools_search_path, &stdlib);
         assert_eq!(
-            Ok(functools_module.file().unwrap()),
+            Ok(functools_file),
             system_path_to_file(&db, &stdlib_functools_path)
         );
     }
@@ -1577,9 +1626,9 @@ mod tests {
 
         let functools_module_name = ModuleName::new_static("functools").unwrap();
         let functools_module = resolve_module(&db, &functools_module_name).unwrap();
-        assert_eq!(functools_module.search_path().unwrap(), &stdlib);
+        assert_eq!(functools_module.search_path(&db).unwrap(), &stdlib);
         assert_eq!(
-            Ok(functools_module.file().unwrap()),
+            Ok(functools_module.file(&db).unwrap()),
             system_path_to_file(&db, stdlib.join("functools.pyi"))
         );
 
@@ -1588,9 +1637,9 @@ mod tests {
         let src_functools_path = src.join("functools.py");
         db.write_file(&src_functools_path, "FOO: int").unwrap();
         let functools_module = resolve_module(&db, &functools_module_name).unwrap();
-        assert_eq!(functools_module.search_path().unwrap(), &src);
+        assert_eq!(functools_module.search_path(&db).unwrap(), &src);
         assert_eq!(
-            Ok(functools_module.file().unwrap()),
+            Ok(functools_module.file(&db).unwrap()),
             system_path_to_file(&db, &src_functools_path)
         );
     }
@@ -1619,9 +1668,9 @@ mod tests {
         let src_functools_path = src.join("functools.py");
 
         let functools_module = resolve_module(&db, &functools_module_name).unwrap();
-        assert_eq!(functools_module.search_path().unwrap(), &src);
+        assert_eq!(functools_module.search_path(&db).unwrap(), &src);
         assert_eq!(
-            Ok(functools_module.file().unwrap()),
+            Ok(functools_module.file(&db).unwrap()),
             system_path_to_file(&db, &src_functools_path)
         );
 
@@ -1632,9 +1681,9 @@ mod tests {
             .unwrap();
         File::sync_path(&mut db, &src_functools_path);
         let functools_module = resolve_module(&db, &functools_module_name).unwrap();
-        assert_eq!(functools_module.search_path().unwrap(), &stdlib);
+        assert_eq!(functools_module.search_path(&db).unwrap(), &stdlib);
         assert_eq!(
-            Ok(functools_module.file().unwrap()),
+            Ok(functools_module.file(&db).unwrap()),
             system_path_to_file(&db, stdlib.join("functools.pyi"))
         );
     }
@@ -1657,11 +1706,11 @@ mod tests {
         let foo_bar_module = resolve_module(&db, &foo_bar_module_name).unwrap();
 
         assert_eq!(
-            foo_module.file().unwrap().path(&db),
+            foo_module.file(&db).unwrap().path(&db),
             &FilePath::system("/x/src/foo/__init__.py")
         );
         assert_eq!(
-            foo_bar_module.file().unwrap().path(&db),
+            foo_bar_module.file(&db).unwrap().path(&db),
             &FilePath::system("/x/src/foo/bar.py")
         );
     }
@@ -1688,7 +1737,7 @@ mod tests {
         let bar_module_name = ModuleName::new_static("bar").unwrap();
         let bar_module = resolve_module(&db, &bar_module_name).unwrap();
         assert_eq!(
-            bar_module.file().unwrap().path(&db),
+            bar_module.file(&db).unwrap().path(&db),
             &FilePath::system("/y/src/bar.py")
         );
     }
@@ -1708,7 +1757,7 @@ mod tests {
         let foo_module = resolve_module(&db, &foo_module_name).unwrap();
 
         assert_eq!(
-            foo_module.file().unwrap().path(&db),
+            foo_module.file(&db).unwrap().path(&db),
             &FilePath::system("/x/y/src/foo.pyi")
         );
     }
@@ -1759,19 +1808,19 @@ not_a_directory
         let spam_module = resolve_module(&db, &spam_module_name).unwrap();
 
         assert_eq!(
-            foo_module.file().unwrap().path(&db),
+            foo_module.file(&db).unwrap().path(&db),
             &FilePath::system("/x/y/src/foo.pyi")
         );
         assert_eq!(
-            a_module.file().unwrap().path(&db),
+            a_module.file(&db).unwrap().path(&db),
             &FilePath::system("/a.py")
         );
         assert_eq!(
-            b_module.file().unwrap().path(&db),
+            b_module.file(&db).unwrap().path(&db),
             &FilePath::system("/baz/b.py")
         );
         assert_eq!(
-            spam_module.file().unwrap().path(&db),
+            spam_module.file(&db).unwrap().path(&db),
             &FilePath::System(site_packages.join("spam/spam.py"))
         );
     }
@@ -1792,14 +1841,14 @@ not_a_directory
 
         let foo_module = resolve_module(&db, &foo_module_name).unwrap();
         assert_eq!(
-            foo_module.file().unwrap().path(&db),
+            foo_module.file(&db).unwrap().path(&db),
             &FilePath::system("/x/src/foo.py")
         );
 
         db.clear_salsa_events();
         let bar_module = resolve_module(&db, &bar_module_name).unwrap();
         assert_eq!(
-            bar_module.file().unwrap().path(&db),
+            bar_module.file(&db).unwrap().path(&db),
             &FilePath::system("/y/src/bar.py")
         );
         let events = db.take_salsa_events();
@@ -1824,7 +1873,7 @@ not_a_directory
         let foo_module_name = ModuleName::new_static("foo").unwrap();
         let foo_module = resolve_module(&db, &foo_module_name).unwrap();
         assert_eq!(
-            foo_module.file().unwrap().path(&db),
+            foo_module.file(&db).unwrap().path(&db),
             &FilePath::system("/x/src/foo.py")
         );
 
@@ -1852,7 +1901,7 @@ not_a_directory
         let foo_module = resolve_module(&db, &foo_module_name).unwrap();
         let src_path = SystemPathBuf::from("/x/src");
         assert_eq!(
-            foo_module.file().unwrap().path(&db),
+            foo_module.file(&db).unwrap().path(&db),
             &FilePath::System(src_path.join("foo.py"))
         );
 
@@ -1923,7 +1972,7 @@ not_a_directory
         let a_module_name = ModuleName::new_static("a").unwrap();
         let a_module = resolve_module(&db, &a_module_name).unwrap();
         assert_eq!(
-            a_module.file().unwrap().path(&db),
+            a_module.file(&db).unwrap().path(&db),
             &editable_install_location
         );
 
@@ -1937,7 +1986,7 @@ not_a_directory
         // second `site-packages` directory
         let a_module = resolve_module(&db, &a_module_name).unwrap();
         assert_eq!(
-            a_module.file().unwrap().path(&db),
+            a_module.file(&db).unwrap().path(&db),
             &system_site_packages_location
         );
     }
@@ -1999,7 +2048,7 @@ not_a_directory
         let a_module = resolve_module(&db, &a_module_name).expect("a.py to resolve");
         assert!(
             a_module
-                .file()
+                .file(&db)
                 .unwrap()
                 .path(&db)
                 .as_str()
@@ -2034,6 +2083,6 @@ not_a_directory
 
         let foo_module_file = File::new(&db, FilePath::System(installed_foo_module));
         let module = file_to_module(&db, foo_module_file).unwrap();
-        assert_eq!(module.search_path().unwrap(), &site_packages);
+        assert_eq!(module.search_path(&db).unwrap(), &site_packages);
     }
 }
