@@ -113,6 +113,9 @@ pub enum PythonVersionSource {
     /// long argument (`--extra-paths`) or `--config key=value`.
     Cli,
 
+    /// The value comes from the Python VS Code extension (the selected interpreter).
+    PythonVSCodeExtension,
+
     /// We fell back to a default value because the value was not specified via the CLI or a config file.
     #[default]
     Default,
