@@ -9,16 +9,20 @@ use crate::semantic_index::place::ScopeId;
 use crate::semantic_index::{
     attribute_scopes, global_scope, place_table, semantic_index, use_def_map,
 };
-use crate::types::call::CallArguments;
+use crate::types::call::{CallArguments, MatchedArgument};
 use crate::types::signatures::Signature;
 use crate::types::{ClassBase, ClassLiteral, DynamicType, KnownClass, KnownInstanceType, Type};
 use crate::{Db, HasType, NameKind, SemanticModel};
-use ruff_db::files::File;
+use ruff_db::files::{File, FileRange};
+use ruff_db::parsed::parsed_module;
 use ruff_python_ast as ast;
 use ruff_python_ast::name::Name;
-use ruff_text_size::TextRange;
+use ruff_text_size::{Ranged, TextRange};
 use rustc_hash::FxHashSet;
 
+pub use resolve_definition::{ResolvedDefinition, map_stub_definition};
+use resolve_definition::{find_symbol_in_scope, resolve_definition};
+
 pub(crate) fn all_declarations_and_bindings<'db>(
     db: &'db dyn Db,
     scope_id: ScopeId<'db>,
@@ -152,7 +156,7 @@ impl<'db> AllMembers<'db> {
                 self.extend_with_type(db, KnownClass::ModuleType.to_instance(db));
                 let module = literal.module(db);
 
-                let Some(file) = module.file() else {
+                let Some(file) = module.file(db) else {
                     return;
                 };
 
@@ -366,7 +370,7 @@ pub fn definition_kind_for_name<'db>(
     let name_str = name.id.as_str();
 
     // Get the scope for this name expression
-    let file_scope = index.try_expression_scope_id(&ast::Expr::Name(name.clone()))?;
+    let file_scope = index.expression_scope_id(&ast::ExprRef::from(name));
 
     // Get the place table for this scope
     let place_table = index.place_table(file_scope);
@@ -399,9 +403,7 @@ pub fn definitions_for_name<'db>(
     let name_str = name.id.as_str();
 
     // Get the scope for this name expression
-    let Some(file_scope) = index.try_expression_scope_id(&ast::Expr::Name(name.clone())) else {
-        return Vec::new();
-    };
+    let file_scope = index.expression_scope_id(&ast::ExprRef::from(name));
 
     let mut all_definitions = Vec::new();
 
@@ -503,6 +505,202 @@ pub fn definitions_for_name<'db>(
     }
 }
 
+/// Returns all resolved definitions for an attribute expression `x.y`.
+/// This function duplicates much of the functionality in the semantic
+/// analyzer, but it has somewhat different behavior so we've decided
+/// to keep it separate for now. One key difference is that this function
+/// doesn't model the descriptor protocol when accessing attributes.
+/// For "go to definition", we want to get the type of the descriptor object
+/// rather than "invoking" its `__get__` or `__set__` method.
+/// If this becomes a maintenance burden in the future, it may be worth
+/// changing the corresponding logic in the semantic analyzer to conditionally
+/// handle this case through the use of mode flags.
+pub fn definitions_for_attribute<'db>(
+    db: &'db dyn Db,
+    file: File,
+    attribute: &ast::ExprAttribute,
+) -> Vec<ResolvedDefinition<'db>> {
+    let name_str = attribute.attr.as_str();
+    let model = SemanticModel::new(db, file);
+
+    let mut resolved = Vec::new();
+
+    // Determine the type of the LHS
+    let lhs_ty = attribute.value.inferred_type(&model);
+    let tys = match lhs_ty {
+        Type::Union(union) => union.elements(db).to_vec(),
+        _ => vec![lhs_ty],
+    };
+
+    // Expand intersections for each subtype into their components
+    let expanded_tys = tys
+        .into_iter()
+        .flat_map(|ty| match ty {
+            Type::Intersection(intersection) => intersection.positive(db).iter().copied().collect(),
+            _ => vec![ty],
+        })
+        .collect::<Vec<_>>();
+
+    for ty in expanded_tys {
+        // Handle modules
+        if let Type::ModuleLiteral(module_literal) = ty {
+            if let Some(module_file) = module_literal.module(db).file(db) {
+                let module_scope = global_scope(db, module_file);
+                for def in find_symbol_in_scope(db, module_scope, name_str) {
+                    resolved.extend(resolve_definition(db, def, Some(name_str)));
+                }
+            }
+            continue;
+        }
+
+        // First, transform the type to its meta type, unless it's already a class-like type.
+        let meta_type = match ty {
+            Type::ClassLiteral(_) | Type::SubclassOf(_) | Type::GenericAlias(_) => ty,
+            _ => ty.to_meta_type(db),
+        };
+        let class_literal = match meta_type {
+            Type::ClassLiteral(class_literal) => class_literal,
+            Type::SubclassOf(subclass) => match subclass.subclass_of().into_class() {
+                Some(cls) => cls.class_literal(db).0,
+                None => continue,
+            },
+            _ => continue,
+        };
+
+        // Walk the MRO: include class and its ancestors, but stop when we find a match
+        'scopes: for ancestor in class_literal
+            .iter_mro(db, None)
+            .filter_map(ClassBase::into_class)
+            .map(|cls| cls.class_literal(db).0)
+        {
+            let class_scope = ancestor.body_scope(db);
+            let class_place_table = crate::semantic_index::place_table(db, class_scope);
+
+            // Look for class-level declarations and bindings
+            if let Some(place_id) = class_place_table.place_id_by_name(name_str) {
+                let use_def = use_def_map(db, class_scope);
+
+                // Check declarations first
+                for decl in use_def.all_reachable_declarations(place_id) {
+                    if let Some(def) = decl.declaration.definition() {
+                        resolved.extend(resolve_definition(db, def, Some(name_str)));
+                        break 'scopes;
+                    }
+                }
+
+                // If no declarations found, check bindings
+                for binding in use_def.all_reachable_bindings(place_id) {
+                    if let Some(def) = binding.binding.definition() {
+                        resolved.extend(resolve_definition(db, def, Some(name_str)));
+                        break 'scopes;
+                    }
+                }
+            }
+
+            // Look for instance attributes in method scopes (e.g., self.x = 1)
+            let file = class_scope.file(db);
+            let index = semantic_index(db, file);
+
+            for function_scope_id in attribute_scopes(db, class_scope) {
+                let place_table = index.place_table(function_scope_id);
+
+                if let Some(place_id) = place_table.place_id_by_instance_attribute_name(name_str) {
+                    let use_def = index.use_def_map(function_scope_id);
+
+                    // Check declarations first
+                    for decl in use_def.all_reachable_declarations(place_id) {
+                        if let Some(def) = decl.declaration.definition() {
+                            resolved.extend(resolve_definition(db, def, Some(name_str)));
+                            break 'scopes;
+                        }
+                    }
+
+                    // If no declarations found, check bindings
+                    for binding in use_def.all_reachable_bindings(place_id) {
+                        if let Some(def) = binding.binding.definition() {
+                            resolved.extend(resolve_definition(db, def, Some(name_str)));
+                            break 'scopes;
+                        }
+                    }
+                }
+            }
+
+            // TODO: Add support for metaclass attribute lookups
+        }
+    }
+
+    resolved
+}
+
+/// Returns definitions for a keyword argument in a call expression.
+/// This resolves the keyword argument to the corresponding parameter(s) in the callable's signature(s).
+pub fn definitions_for_keyword_argument<'db>(
+    db: &'db dyn Db,
+    file: File,
+    keyword: &ast::Keyword,
+    call_expr: &ast::ExprCall,
+) -> Vec<ResolvedDefinition<'db>> {
+    let model = SemanticModel::new(db, file);
+    let func_type = call_expr.func.inferred_type(&model);
+
+    let Some(keyword_name) = keyword.arg.as_ref() else {
+        return Vec::new();
+    };
+    let keyword_name_str = keyword_name.as_str();
+
+    let mut resolved_definitions = Vec::new();
+
+    if let Some(Type::Callable(callable_type)) = func_type.into_callable(db) {
+        let signatures = callable_type.signatures(db);
+
+        // For each signature, find the parameter with the matching name
+        for signature in signatures {
+            if let Some((_param_index, _param)) =
+                signature.parameters().keyword_by_name(keyword_name_str)
+            {
+                if let Some(function_definition) = signature.definition() {
+                    let function_file = function_definition.file(db);
+                    let module = parsed_module(db, function_file).load(db);
+                    let def_kind = function_definition.kind(db);
+
+                    if let DefinitionKind::Function(function_ast_ref) = def_kind {
+                        let function_node = function_ast_ref.node(&module);
+
+                        if let Some(parameter_range) =
+                            find_parameter_range(&function_node.parameters, keyword_name_str)
+                        {
+                            resolved_definitions.push(ResolvedDefinition::FileWithRange(
+                                FileRange::new(function_file, parameter_range),
+                            ));
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    resolved_definitions
+}
+
+/// Find the definitions for a symbol imported via `from x import y as z` statement.
+/// This function handles the case where the cursor is on the original symbol name `y`.
+/// Returns the same definitions as would be found for the alias `z`.
+pub fn definitions_for_imported_symbol<'db>(
+    db: &'db dyn Db,
+    file: File,
+    import_node: &ast::StmtImportFrom,
+    symbol_name: &str,
+) -> Vec<ResolvedDefinition<'db>> {
+    let mut visited = FxHashSet::default();
+    resolve_definition::resolve_from_import_definitions(
+        db,
+        file,
+        import_node,
+        symbol_name,
+        &mut visited,
+    )
+}
+
 /// Details about a callable signature for IDE support.
 #[derive(Debug, Clone)]
 pub struct CallSignatureDetails<'db> {
@@ -527,7 +725,7 @@ pub struct CallSignatureDetails<'db> {
 
     /// Mapping from argument indices to parameter indices. This helps
     /// determine which parameter corresponds to which argument position.
-    pub argument_to_parameter_mapping: Vec<Option<usize>>,
+    pub argument_to_parameter_mapping: Vec<MatchedArgument>,
 }
 
 /// Extract signature details from a function call expression.
@@ -544,7 +742,10 @@ pub fn call_signature_details<'db>(
 
     // Use into_callable to handle all the complex type conversions
     if let Some(callable_type) = func_type.into_callable(db) {
-        let call_arguments = CallArguments::from_arguments(&call_expr.arguments);
+        let call_arguments =
+            CallArguments::from_arguments(db, &call_expr.arguments, |_, splatted_value| {
+                splatted_value.inferred_type(&model)
+            });
         let bindings = callable_type.bindings(db).match_parameters(&call_arguments);
 
         // Extract signature details from all callable bindings
@@ -563,7 +764,7 @@ pub fn call_signature_details<'db>(
                     parameter_label_offsets,
                     parameter_names,
                     definition: signature.definition(),
-                    argument_to_parameter_mapping: binding.argument_to_parameter_mapping().to_vec(),
+                    argument_to_parameter_mapping: binding.argument_matches().to_vec(),
                 }
             })
             .collect()
@@ -573,31 +774,57 @@ pub fn call_signature_details<'db>(
     }
 }
 
+/// Find the text range of a specific parameter in function parameters by name.
+/// Only searches for parameters that can be addressed by name in keyword arguments.
+fn find_parameter_range(parameters: &ast::Parameters, parameter_name: &str) -> Option<TextRange> {
+    // Check regular positional and keyword-only parameters
+    parameters
+        .args
+        .iter()
+        .chain(&parameters.kwonlyargs)
+        .find(|param| param.parameter.name.as_str() == parameter_name)
+        .map(|param| param.parameter.name.range())
+}
+
 mod resolve_definition {
     //! Resolves an Import, `ImportFrom` or `StarImport` definition to one or more
     //! "resolved definitions". This is done recursively to find the original
     //! definition targeted by the import.
 
-    use ruff_db::files::File;
-    use ruff_db::parsed::parsed_module;
+    use indexmap::IndexSet;
+    use ruff_db::files::{File, FileRange};
+    use ruff_db::parsed::{ParsedModuleRef, parsed_module};
     use ruff_python_ast as ast;
+    use ruff_text_size::{Ranged, TextRange};
     use rustc_hash::FxHashSet;
+    use tracing::trace;
 
+    use crate::module_resolver::file_to_module;
     use crate::semantic_index::definition::{Definition, DefinitionKind};
-    use crate::semantic_index::place::ScopeId;
-    use crate::semantic_index::{global_scope, place_table, use_def_map};
-    use crate::{Db, ModuleName, resolve_module};
+    use crate::semantic_index::place::{NodeWithScopeKind, ScopeId};
+    use crate::semantic_index::{global_scope, place_table, semantic_index, use_def_map};
+    use crate::{Db, ModuleName, resolve_module, resolve_real_module};
 
-    /// Represents the result of resolving an import to either a specific definition or a module file.
+    /// Represents the result of resolving an import to either a specific definition or
+    /// a specific range within a file.
     /// This enum helps distinguish between cases where an import resolves to:
     /// - A specific definition within a module (e.g., `from os import path` -> definition of `path`)
-    /// - An entire module file (e.g., `import os` -> the `os` module file itself)
+    /// - A specific range within a file, sometimes an empty range at the top of the file
     #[derive(Debug, Clone, PartialEq, Eq)]
     pub enum ResolvedDefinition<'db> {
         /// The import resolved to a specific definition within a module
         Definition(Definition<'db>),
-        /// The import resolved to an entire module file
-        ModuleFile(File),
+        /// The import resolved to a file with a specific range
+        FileWithRange(FileRange),
+    }
+
+    impl<'db> ResolvedDefinition<'db> {
+        fn file(&self, db: &'db dyn Db) -> File {
+            match self {
+                ResolvedDefinition::Definition(definition) => definition.file(db),
+                ResolvedDefinition::FileWithRange(file_range) => file_range.file(),
+            }
+        }
     }
 
     /// Resolve import definitions to their targets.
@@ -651,13 +878,16 @@ mod resolve_definition {
                     return Vec::new(); // Module not found, return empty list
                 };
 
-                let Some(module_file) = resolved_module.file() else {
+                let Some(module_file) = resolved_module.file(db) else {
                     return Vec::new(); // No file for module, return empty list
                 };
 
                 // For simple imports like "import os", we want to navigate to the module itself.
                 // Return the module file directly instead of trying to find definitions within it.
-                vec![ResolvedDefinition::ModuleFile(module_file)]
+                vec![ResolvedDefinition::FileWithRange(FileRange::new(
+                    module_file,
+                    TextRange::default(),
+                ))]
             }
 
             DefinitionKind::ImportFrom(import_from_def) => {
@@ -692,7 +922,7 @@ mod resolve_definition {
     }
 
     /// Helper function to resolve import definitions for `ImportFrom` and `StarImport` cases.
-    fn resolve_from_import_definitions<'db>(
+    pub(crate) fn resolve_from_import_definitions<'db>(
         db: &'db dyn Db,
         file: File,
         import_node: &ast::StmtImportFrom,
@@ -709,7 +939,7 @@ mod resolve_definition {
             let Some(resolved_module) = resolve_module(db, &module_name) else {
                 return Vec::new();
             };
-            resolved_module.file()
+            resolved_module.file(db)
         };
 
         let Some(module_file) = module_file else {
@@ -739,14 +969,14 @@ mod resolve_definition {
         db: &'db dyn Db,
         scope: ScopeId<'db>,
         symbol_name: &str,
-    ) -> Vec<Definition<'db>> {
+    ) -> IndexSet<Definition<'db>> {
         let place_table = place_table(db, scope);
         let Some(place_id) = place_table.place_id_by_name(symbol_name) else {
-            return Vec::new();
+            return IndexSet::new();
         };
 
         let use_def_map = use_def_map(db, scope);
-        let mut definitions = Vec::new();
+        let mut definitions = IndexSet::new();
 
         // Get all definitions (both bindings and declarations) for this place
         let bindings = use_def_map.all_reachable_bindings(place_id);
@@ -754,19 +984,223 @@ mod resolve_definition {
 
         for binding in bindings {
             if let Some(def) = binding.binding.definition() {
-                definitions.push(def);
+                definitions.insert(def);
             }
         }
 
         for declaration in declarations {
             if let Some(def) = declaration.declaration.definition() {
-                definitions.push(def);
+                definitions.insert(def);
             }
         }
 
         definitions
     }
-}
 
-pub use resolve_definition::ResolvedDefinition;
-use resolve_definition::{find_symbol_in_scope, resolve_definition};
+    /// Given a definition that may be in a stub file, find the "real" definition in a non-stub.
+    #[tracing::instrument(skip_all)]
+    pub fn map_stub_definition<'db>(
+        db: &'db dyn Db,
+        def: &ResolvedDefinition<'db>,
+    ) -> Option<Vec<ResolvedDefinition<'db>>> {
+        trace!("Stub mapping definition...");
+        // If the file isn't a stub, this is presumably the real definition
+        let stub_file = def.file(db);
+        if !stub_file.is_stub(db) {
+            trace!("File isn't a stub, no stub mapping to do");
+            return None;
+        }
+
+        // It's definitely a stub, so now rerun module resolution but with stubs disabled.
+        let stub_module = file_to_module(db, stub_file)?;
+        trace!("Found stub module: {}", stub_module.name(db));
+        let real_module = resolve_real_module(db, stub_module.name(db))?;
+        trace!("Found real module: {}", real_module.name(db));
+        let real_file = real_module.file(db)?;
+        trace!("Found real file: {}", real_file.path(db));
+
+        // A definition has a "Definition Path" in a file made of nested definitions (~scopes):
+        //
+        // ```
+        // class myclass:  # ./myclass
+        //     def some_func(args: bool):  # ./myclass/some_func
+        //                 # ^~~~ ./myclass/other_func/args/
+        // ```
+        //
+        // So our heuristic goal here is to compute a Definition Path in the stub file
+        // and then resolve the same Definition Path in the real file.
+        //
+        // NOTE: currently a path component is just a str, but in the future additional
+        // disambiguators (like "is a class def") could be added if needed.
+        let mut path = Vec::new();
+        let stub_parsed;
+        let stub_ref;
+        match *def {
+            ResolvedDefinition::Definition(definition) => {
+                stub_parsed = parsed_module(db, stub_file);
+                stub_ref = stub_parsed.load(db);
+
+                // Get the leaf of the path (the definition itself)
+                let leaf = definition_path_component_for_leaf(db, &stub_ref, definition)
+                    .map_err(|()| {
+                        trace!("Found unsupported DefinitionKind while stub mapping, giving up");
+                    })
+                    .ok()?;
+                path.push(leaf);
+
+                // Get the ancestors of the path (all the definitions we're nested under)
+                let index = semantic_index(db, stub_file);
+                for (_scope_id, scope) in index.ancestor_scopes(definition.file_scope(db)) {
+                    let node = scope.node();
+                    let component = definition_path_component_for_node(&stub_ref, node)
+                        .map_err(|()| {
+                            trace!("Found unsupported NodeScopeKind while stub mapping, giving up");
+                        })
+                        .ok()?;
+                    if let Some(component) = component {
+                        path.push(component);
+                    }
+                }
+                trace!("Built Definition Path: {path:?}");
+            }
+            ResolvedDefinition::FileWithRange(file_range) => {
+                return if file_range.range() == TextRange::default() {
+                    trace!(
+                        "Found module mapping: {} => {}",
+                        stub_file.path(db),
+                        real_file.path(db)
+                    );
+                    // This is just a reference to a module, no need to do paths
+                    Some(vec![ResolvedDefinition::FileWithRange(FileRange::new(
+                        real_file,
+                        TextRange::default(),
+                    ))])
+                } else {
+                    // Not yet implemented -- in this case we want to recover something like a Definition
+                    // and build a Definition Path, but this input is a bit too abstract for now.
+                    trace!("Found arbitrary FileWithRange by stub mapping, giving up");
+                    None
+                };
+            }
+        }
+
+        // Walk down the Definition Path in the real file
+        let mut definitions = Vec::new();
+        let index = semantic_index(db, real_file);
+        let real_parsed = parsed_module(db, real_file);
+        let real_ref = real_parsed.load(db);
+        // Start our search in the module (global) scope
+        let mut scopes = vec![global_scope(db, real_file)];
+        while let Some(component) = path.pop() {
+            trace!("Traversing definition path component: {}", component);
+            // We're doing essentially a breadth-first traversal of the definitions.
+            // If ever we find multiple matching scopes for a component, we need to continue
+            // walking down each of them to try to resolve the path. Here we loop over
+            // all the scopes at the current level of search.
+            for scope in std::mem::take(&mut scopes) {
+                if path.is_empty() {
+                    // We're at the end of the path, everything we find here is the final result
+                    definitions.extend(
+                        find_symbol_in_scope(db, scope, component)
+                            .into_iter()
+                            .map(ResolvedDefinition::Definition),
+                    );
+                } else {
+                    // We're in the middle of the path, look for scopes that match the current component
+                    for (child_scope_id, child_scope) in index.child_scopes(scope.file_scope_id(db))
+                    {
+                        let scope_node = child_scope.node();
+                        if let Ok(Some(real_component)) =
+                            definition_path_component_for_node(&real_ref, scope_node)
+                        {
+                            if real_component == component {
+                                scopes.push(child_scope_id.to_scope_id(db, real_file));
+                            }
+                        }
+                        scope.node(db);
+                    }
+                }
+            }
+            trace!(
+                "Found {} scopes and {} definitions",
+                scopes.len(),
+                definitions.len()
+            );
+        }
+        if definitions.is_empty() {
+            trace!("No definitions found in real file, stub mapping failed");
+            None
+        } else {
+            trace!("Found {} definitions from stub mapping", definitions.len());
+            Some(definitions)
+        }
+    }
+
+    /// Computes a "Definition Path" component for an internal node of the definition path.
+    ///
+    /// See [`map_stub_definition`][] for details.
+    fn definition_path_component_for_node<'parse>(
+        parsed: &'parse ParsedModuleRef,
+        node: &NodeWithScopeKind,
+    ) -> Result<Option<&'parse str>, ()> {
+        let component = match node {
+            NodeWithScopeKind::Module => {
+                // This is just implicit, so has no component
+                return Ok(None);
+            }
+            NodeWithScopeKind::Class(class) => class.node(parsed).name.as_str(),
+            NodeWithScopeKind::Function(func) => func.node(parsed).name.as_str(),
+            NodeWithScopeKind::TypeAlias(_)
+            | NodeWithScopeKind::ClassTypeParameters(_)
+            | NodeWithScopeKind::FunctionTypeParameters(_)
+            | NodeWithScopeKind::TypeAliasTypeParameters(_)
+            | NodeWithScopeKind::Lambda(_)
+            | NodeWithScopeKind::ListComprehension(_)
+            | NodeWithScopeKind::SetComprehension(_)
+            | NodeWithScopeKind::DictComprehension(_)
+            | NodeWithScopeKind::GeneratorExpression(_) => {
+                // Not yet implemented
+                return Err(());
+            }
+        };
+        Ok(Some(component))
+    }
+
+    /// Computes a "Definition Path" component for a leaf node of the definition path.
+    ///
+    /// See [`map_stub_definition`][] for details.
+    fn definition_path_component_for_leaf<'parse>(
+        db: &dyn Db,
+        parsed: &'parse ParsedModuleRef,
+        definition: Definition,
+    ) -> Result<&'parse str, ()> {
+        let component = match definition.kind(db) {
+            DefinitionKind::Function(func) => func.node(parsed).name.as_str(),
+            DefinitionKind::Class(class) => class.node(parsed).name.as_str(),
+            DefinitionKind::TypeAlias(_)
+            | DefinitionKind::Import(_)
+            | DefinitionKind::ImportFrom(_)
+            | DefinitionKind::StarImport(_)
+            | DefinitionKind::NamedExpression(_)
+            | DefinitionKind::Assignment(_)
+            | DefinitionKind::AnnotatedAssignment(_)
+            | DefinitionKind::AugmentedAssignment(_)
+            | DefinitionKind::For(_)
+            | DefinitionKind::Comprehension(_)
+            | DefinitionKind::VariadicPositionalParameter(_)
+            | DefinitionKind::VariadicKeywordParameter(_)
+            | DefinitionKind::Parameter(_)
+            | DefinitionKind::WithItem(_)
+            | DefinitionKind::MatchPattern(_)
+            | DefinitionKind::ExceptHandler(_)
+            | DefinitionKind::TypeVar(_)
+            | DefinitionKind::ParamSpec(_)
+            | DefinitionKind::TypeVarTuple(_) => {
+                // Not yet implemented
+                return Err(());
+            }
+        };
+
+        Ok(component)
+    }
+}
